#! /usr/bin/perl
# 
# fortune.pl - RFC865 compliant client in PERL.
#
# Copyright (C) 2000 Afghan Hound Software
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

use strict;
use IO::Socket;
use Getopt::Std;
use vars qw/ $opt_h $opt_v $opt_l $opt_s $opt_p $opt_t $opt_w /;

my ($env_host,$env_port,$def_host,$def_port,$type,$out);
my ($arg_host,$arg_port,$proto,$remote,$host,$port);

getopts('hvlws:p:t:');

$def_host = "127.0.0.1";
$def_port = "qotd";
$proto    = "tcp";
$type     = SOCK_STREAM;

if ($opt_v)
  {
    print "Afghan fortune.pl version 1.0.0. Copyright (C) 2000 Afghan Hound.\n\n";
    print "  *** This program comes with ABSOLUTELY NO WARRANTY. *** \n\n";
    print "You may redistribute copies of Afghan fortune.pl under the terms\n";
    print "of the GNU General Public License. For more information about\n";
    print "these matters, see the file named COPYING.\n";
    exit(0);
  }
if ($opt_h)
  {
    print "Afghan fortune.pl version 1.0.0. Copyright (C) 2000 Afghan Hound.\n";
    print "Usage:\n";
    print "     -s hostname : specify QOTD server hostname. (Default=$def_host)\n";
    print "     -p port     : specify QOTD server port number. (Default=$def_port)\n";
    print "     -t [tcp/udp]: Protocol to use. (Default=$proto).\n";
    print "     -l          : Print extra blank lines before and after quote.\n";
    print "     -w          : Wait for 0.05761 seconds per character before exiting.\n";
    print "     -h          : print this help.\n";	
    print "     -v          : print version information.\n";
    exit(0);
  }

$env_host = $ENV{"QOTD_HOST"};
$env_port = $ENV{"QOTD_PORT"};
$arg_host = $opt_s;
$arg_port = $opt_p;

(($opt_t eq "udp")||(!($opt_t))||($opt_t eq "tcp")) 
  or (($opt_t=uc($opt_t))&&
      (die ("$opt_t is an unsupported protocol")));

($opt_t eq "udp")&&($proto="udp")&&($type=SOCK_DGRAM);

$host = $def_host;
($env_host)&&($host = $env_host);
($arg_host)&&($host = $arg_host);

$port = $def_port;
($env_port)&&($port = $env_port);
($arg_port)&&($port = $arg_port);


$remote = IO::Socket::INET->new(
			Proto    => $proto,
			PeerAddr => $host,
			PeerPort => $port,
			Type     => $type,);
$out = "";
if ($opt_t eq "udp")
  {
    $remote->send("\n");
    $remote->recv($out,9000,0);
    if (!($out))
      {
	print "Connection Refused: udp/$port \@ $host\n";
	exit(1);
      }
  }
else
  {
    if (!($remote))
      {
	print "Connection Refused: tcp/$port \@ $host\n";
	exit(1);	
      }
    while(<$remote>){$out.=$_;}
  }


($opt_l)&&(print "\n");
print $out;
($opt_l)&&(print "\n");
($opt_w)&&(sleep(length($out)*0.05761));
exit(0);

